(* Sample program for use of unit FileTools:
   Copy, pack and unpack files, create zip archives
   Optional use of encryption

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Nov. 2005
   Vers. 2 - Apr. 2006
   Vers. 3 - May  2007
   *)

unit EZipMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, XPMan, StdCtrls, ComCtrls, Buttons, FtGlobal, ExtFileTools, AesLib;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    VerzBtn: TSpeedButton;
    Label2: TLabel;
    FileBtn: TSpeedButton;
    edtDest: TEdit;
    edtFiles: TEdit;
    OpenDialog: TOpenDialog;
    ProgressBar: TProgressBar;
    Label3: TLabel;
    trbPrior: TTrackBar;
    btnPause: TSpeedButton;
    btnThCopy: TButton;
    btnGUnzip: TButton;
    btnThZip: TButton;
    btnQuit: TButton;
    XPManifest: TXPManifest;
    cbxEncrypt: TCheckBox;
    Label4: TLabel;
    edtPwd: TEdit;
    cbxCompress: TCheckBox;
    cbxVerify: TCheckBox;
    procedure btnThCopyClick(Sender: TObject);
    procedure btnGUnzipClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnQuitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnThZipClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure VerzBtnClick(Sender: TObject);
    procedure trbPriorChange(Sender: TObject);
  private
    { Private-Deklarationen }
    IniName     : string;
    CopyThread  : TCopyThread;
    Zip         : TZipWrite;
    nr,na       : int64;

    procedure ShowStatus (Start : boolean);
    procedure ShowProgress (AAction : TFileAction; Increment : int64);
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses IniFiles, ShellDirDlg, JrUtils, Crypt;

const
  BufSize = 256*1024;
  MaxUserPwdLength = 64;

{ ------------------------------------------------------------------- }
function ReadNxtStr (var s   : String;
                     Del     : char) : string;
var
  i : integer;
begin
  if length(s)>0 then begin
    i:=pos (Del,s);
    if i=0 then i:=succ(length(s));
    Result:=copy(s,1,pred(i));
    delete(s,1,i);
    end
  else Result:='';
  end;

const
  PwdKey = '6B93AFCD27B0D081F5E7E2815D25C4C3416026737D163AF1'; //54AFqnU2

{ ------------------------------------------------------------------- }
procedure TForm1.FormCreate(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniName:=ChangeFileExt(Application.ExeName,'.ini');
  IniFile:=TIniFile.Create (iniName);
  with IniFile do begin
    edtFiles.Text:=ReadString('Config','Filenames','');
    edtDest.Text:=ReadString('Config','Directory','');
    edtPwd.Text:=DecryptPwdString(DecodeString(PwdKey),ReadString('Config','Password',''),teAscii85);
    Free;
    end;
  CopyThread:=nil;
  end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniFile:=TIniFile.Create (iniName);
  with IniFile do begin
    WriteString('Config','Filenames',edtFiles.Text);
    WriteString('Config','Directory',edtDest.Text);
    WriteString('Config','Password',EncryptPwdString(DecodeString(PwdKey),edtPwd.Text,MaxUserPwdLength,teAscii85));
    Free;
    end;
  end;

procedure TForm1.ShowStatus (Start : boolean);
begin
  if Start then begin
    ProgressBar.Position:=0;
    btnPause.Show;
    Screen.Cursor:=crHourglass;
    Application.ProcessMessages;
    end
  else begin
    btnPause.Hide;
    Screen.Cursor:=crDefault;
    CopyThread:=nil;
    end;
  end;

procedure TForm1.ShowProgress (AAction : TFileAction; Increment : int64);
begin
  nr:=nr+Increment;
  with ProgressBar do Position:=round(nr/na*100);
  end;

procedure TForm1.btnThCopyClick(Sender: TObject);
var
  s,t,v       : string;
  Encryption  : TEncryption;
begin
  s:=edtFiles.Text;
  na:=0; nr:=0;
  repeat
    na:=na+LongFileSize(ReadNxtStr(s,','));
    until length(s)=0;
  ShowStatus(true);
  s:=edtFiles.Text;
  if cbxEncrypt.Checked then begin
    Encryption:=TEncryption.Create(edtPwd.Text,0,BufSize);
    end
  else Encryption:=nil;
  repeat
    t:=ReadNxtStr(s,',');
    v:=ExcludeTrailingPathDelimiter(edtDest.Text)+ExtractFileName(t);
    if cbxCompress.Checked then v:=v+GzExt;
    with TWinCopyThread.Create(t,v,Encryption,cbxCompress.Checked,cbxVerify.Checked,
           false,BufSize,TThreadPriority(trbPrior.Position)) do begin
      OnProgress:=ShowProgress;
      repeat Application.ProcessMessages until Done;
      if ErrorCode<>errOK then
        MessageDlg (GetCopyErrMsg(ErrorCode)+#13'File: '+t,mtError,[mbOK],0);
      Free;
      end;
    until length(s)=0;
  ShowStatus(false);
  end;

procedure TForm1.btnGUnzipClick(Sender: TObject);
var
  s,t        : string;
  FileInfo   : TGzFileInfo;
  Decryption : TEncryption;
begin
  s:=edtFiles.Text;
  na:=0; nr:=0;
  repeat
    if GzFileInfo(ReadNxtStr(s,','),FileInfo) then na:=na+FileInfo.USize;
    until length(s)=0;
  ShowStatus(true);
  s:=edtFiles.Text;
  if cbxEncrypt.Checked then Decryption:=TEncryption.Create(edtPwd.Text,0,BufSize)
  else Decryption:=nil;
  repeat
    t:=ReadNxtStr(s,',');
    CopyThread:=TGUnZipThread.Create(t,edtDest.Text,
       Decryption,false,BufSize,TThreadPriority(trbPrior.Position));
    with CopyThread do begin
      OnProgress:=ShowProgress;
      repeat Application.ProcessMessages until Done;
      if ErrorCode<>errOK then
         MessageDlg (GetCopyErrMsg(ErrorCode)+#13'File: '+t,mtError,[mbOK],0);
      Free;
      end;
    until length(s)=0;
  if cbxEncrypt.Checked then Decryption.Free;
  ShowStatus(false);
  end;

procedure TForm1.btnThZipClick(Sender: TObject);
var
  s,t : string;
begin
  s:=edtFiles.Text;
  na:=0; nr:=0;
  repeat
    na:=na+LongFileSize(ReadNxtStr(s,','));
    until length(s)=0;
  ShowStatus(true);
  s:=edtFiles.Text;
  if cbxEncrypt.Checked then t:=edtPwd.Text else t:='';
  Zip:=TZipWrite.Create(ExcludeTrailingPathDelimiter(edtDest.Text)+'Test.zip',
       ExtractFilePath(ReadNxtStr(s,',')),t,0,BufSize);
  with Zip do begin
    OnProgress:=ShowProgress;
    s:=edtFiles.Text;
    repeat
      t:=AnsiDequotedStr(ReadNxtStr(s,','),'"');
      try
        AddFile(t);
      except
        on E:EZipError do MessageDlg (E.Message+#13'File: '+t,mtError,[mbOK],0);
        end;
      until length(s)=0;
    try
      WriteDirectory;
    except
      on E:EZipError do MessageDlg (E.Message+#13'File: '+t,mtError,[mbOK],0);
      end;
    if cbxVerify.Checked then begin
      try
        Verify;
      except
        on E:EZipError do MessageDlg (E.Message+#13'File: '+t,mtError,[mbOK],0);
        end;
      end;
    Free;
    end;
  ShowStatus(false);
  end;

{ ------------------------------------------------------------------- }
procedure TForm1.btnPauseClick(Sender: TObject);
begin
  if assigned(CopyThread) then begin
    with CopyThread do if btnPause.Down then Suspend else Resume;
    end
  else with Zip do if btnPause.Down then Suspend else Resume;
  end;

procedure TForm1.trbPriorChange(Sender: TObject);
begin
  if assigned(CopyThread) then CopyThread.Priority:=TThreadPriority(trbPrior.Position)
  else Zip.ThreadPriority:=TThreadPriority(trbPrior.Position);
  end;

procedure TForm1.btnQuitClick(Sender: TObject);
begin
  Close;
  end;

procedure TForm1.FileBtnClick(Sender: TObject);
var
  s : string;
begin
  with Opendialog do begin
    s:=edtFiles.Text;
    InitialDir:=ExtractFilePath(ReadNxtStr(s,','));
    if Execute then begin
      if Files.Count>0 then edtFiles.Text:=Files.CommaText
      else edtFiles.Text:=FileName;
      end;
    end;
  end;

procedure TForm1.VerzBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=edtDest.Text;
  if ShellDirDialog.Execute ('Select destination folder',true,true,GetCurrentDir,s) then
    edtDest.Text:=s;
  end;

end.
